import 'package:flutter/foundation.dart';
import 'package:listenable/listenable.dart';

part 'counters.g.dart';

class Counter = _Counter with _Counter$Listenable;

@ListenableModel()
abstract class _Counter {
  @ListenableProperty()
  final String name;
  @ListenableProperty(updatesModel: false)
  int _count = 0;

  _Counter(this.name);

  void increment() {
    _count++;
  }

  void decrement() {
    _count--;
  }
}

class CountersWorld = _CountersWorld with _CountersWorld$Listenable;

@ListenableModel()
abstract class _CountersWorld {
  @ListenableProperty()
  List<Counter> counters = [];

  void addCounter(String name) {
    final newCounter = Counter(name)
      .._count = 0;
    counters.add(newCounter);
    counters = List.of(counters);
  }

  void removeCounter(Counter counter) {
    counters.remove(counter);
    counters = List.of(counters);
  }
}

class CountersController = _CountersController
    with _CountersController$Listenable;

@ListenableModel()
abstract class _CountersController {
  @ListenableProperty()
  String name = "";
  CountersWorld countersWorld = CountersWorld();

  void addCounter() {
    countersWorld.addCounter(name);
  }

  void remove(Counter counter) {
    countersWorld.removeCounter(counter);
  }
}
