import 'package:example/counters.dart';
import 'package:flutter/material.dart';

void main() {
  runApp(CountersApp());
}

class CountersApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    final controller = CountersController();
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        primarySwatch: Colors.blue,
      ),
      home: Scaffold(
        appBar: AppBar(
          title: Text("Listenable Counter Example"),
        ),
        body: Center(
          child: SafeArea(
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Column(
                children: <Widget>[
                  CounterForm(controller),
                  Container(height: 1, color: Colors.lightBlueAccent),
                  CountersWidget(controller: controller),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }
}

class CounterForm extends StatelessWidget {
  final CountersController controller;

  CounterForm(this.controller);

  @override
  Widget build(BuildContext context) {
    final nameController = TextEditingController();
    nameController.addListener(() => controller.name = nameController.text);
    return SizedBox(
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          Expanded(
            flex: 3,
            child: TextField(
                controller: nameController,
                decoration: InputDecoration(labelText: 'Counter Name')),
          ),
          Expanded(
            flex: 1,
            child: Padding(
              padding: const EdgeInsets.symmetric(vertical: 16.0),
              child: RaisedButton(
                onPressed: () {
                  controller.addCounter();
                },
                child: Text(
                  'Add Counter',
                  textAlign: TextAlign.center,
                ),
              ),
            ),
          )
        ],
      ),
    );
  }
}

class CountersWidget extends StatelessWidget {
  final CountersController controller;

  const CountersWidget({Key key, this.controller}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Expanded(
        child: ValueListenableBuilder<CountersWorld>(
      valueListenable: controller.countersWorld.listenable,
      builder: (context, world, _) {
        return ListView.builder(
          itemCount: world.counters.length,
          itemBuilder: (context, index) =>
              new CounterTile(controller, world.counters[index]),
        );
      },
    ));
  }
}

class CounterTile extends StatelessWidget {
  final CountersController controller;
  final Counter _counter;

  CounterTile(this.controller, this._counter);

  @override
  Widget build(BuildContext context) {
    return Dismissible(
      key: ValueKey(_counter),
      onDismissed: (direction) {
        controller.remove(_counter);
      },
      child: ListTile(
        title: SizedBox(
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              Expanded(
                flex: 4,
                child: ValueListenableBuilder<int>(
                    valueListenable: _counter.countListenable,
                    builder: (context, count, _) => Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            Text(
                              '=>[$count]',
                              style: TextStyle(
                                  fontSize: 35, fontWeight: FontWeight.bold),
                            ),
                            Text('${_counter.name}'),
                          ],
                        )),
              ),
              Expanded(
                child: IconButton(
                  icon: Icon(Icons.add_box),
                  onPressed: () {
                    _counter.increment();
                  },
                ),
              ),
              Expanded(
                child: IconButton(
                  icon: Icon(Icons.indeterminate_check_box),
                  onPressed: () {
                    _counter.decrement();
                  },
                ),
              )
            ],
          ),
        ),
      ),
    );
  }
}
